from numpy import genfromtxt,isnan
import pandas as pd
from itertools import product
import os
from glob import glob
import re
import logging
import numpy as np
logging.basicConfig(level=logging.DEBUG)
LOG = logging.getLogger(__name__)

color_sequence = [('blue','grey'),
                  ('grey','blue'),
                  ('green','yellow'),
                  ('yellow','green'),
                  ('black','white'),
                  ('white','black'),
                  ('red','black'),
                  ('black','red')]
color_temps = {
    'grey': 360,
    'red': 337,
    'green': 323,
    'yellow': 288,
    'blue': 254,
    'white': 233,
    'black': 222
}
temp_colors = {v: k for k, v in color_temps.items()}


def read_cal_file(path,color1,color2,**kwargs):
    """
    Read a single calibration csv file
    :param path: path to csv file
    :param color1: color of the resistors in channels 0-3
    :param color2: color of the resistors in channels 4-7
    :param kwargs:
    :return: Dataframe
    """
    if os.path.isfile(path):
        try:
            data = pd.read_csv(path, header=None, usecols=range(1,41))
        except pd.errors.EmptyDataError:
            return pd.DataFrame([])
        mi = list(zip(*product(range(8),['raw_count', 'count_dev', 'corrected_count', 'resistance', 'temperature'])))
        mi.insert(0,[color1]*20+[color2]*20)
        data.columns = pd.MultiIndex.from_arrays(mi,names=['color', 'channel', 'field'])
        if data.isnull().all().all():
            LOG.warning('%s is empty',path)
        # Make all non-number values NaN, then use dropna() to make the shape of array incorrect
        # if NaN values are pressent. This will inform user of invalid data in the notebook.
        return data.apply(pd.to_numeric, errors='coerce').dropna()


def read_files(base_path,t):
    files = {}
    # For csv file in directory
    for path in glob(os.path.join(base_path, '*.csv')) + glob(os.path.join(base_path, '*.CSV')):
        basename = os.path.basename(path).lower()
        # if properly named
        if re.match("{}0([1-8]).csv".format(t), basename, flags=re.I):
            LOG.info('Reading %s', basename)
            # Note the file number
            file_number = int(basename[4])
            # Stick the dataframe in a dictionary
            files[file_number] = read_cal_file(path, *color_sequence[file_number - 1])
    if files == {}:
        LOG.warning('Could not find any %s files in %s', t.upper(), os.path.abspath(base_path))
    # Concatenate all of the results along the time axis, order in time axis doesn't matter
    # We just need records for all of the samples
    return pd.concat(files.values(), axis=0, ignore_index=True)


def read_cal_files(base_path):
    """
    Read all calibration files in a calibration run and concatenate the results.
    The output from all of the cal files is automatically aligned by color and channel during concatenation.
    :param base_path: path to directory containing cal%d.csv files
    :return: MultiIndex DataFrame, much of which is NaNs, but contains all the required data
    """
    return read_files(base_path,'cal')

def read_chk_files(base_path):
    return read_files(base_path,'chk')


read_chk_file = read_cal_file
